import { useEffect } from 'react';
/**
 * Usage: Focuses on the element based on the given condition
 * @example useElementFocus(elementRef, {delay: 300, shouldFocus, triggerKey: 'space'})
 * @param elementRef ref of the element to focus
 * @param shouldFocus boolean value to specify if the element should be focused or not
 * @param delay focus on the given ref after some delay in milliseconds.
 * @param triggerKey click on the ref element if this key is pressed
 */
const useElementFocus = (elementOrRef, { shouldFocus = true, delay = 0, triggerKey } = {}) => {
    const getElement = () => elementOrRef && typeof elementOrRef === 'object' && 'current' in elementOrRef
        ? elementOrRef.current
        : elementOrRef;
    const triggerFocus = () => {
        const el = getElement();
        if (el)
            el.focus();
    };
    useEffect(() => {
        const el = getElement();
        if (!shouldFocus || !el || triggerKey)
            return;
        if (delay > 0) {
            const timer = setTimeout(triggerFocus, delay);
            return () => clearTimeout(timer);
        }
        triggerFocus();
    }, [shouldFocus, elementOrRef, triggerKey, delay]);
    useEffect(() => {
        const el = getElement();
        if (!shouldFocus || !el || !triggerKey)
            return;
        const handler = (e) => {
            if (e.key === triggerKey) {
                if (delay > 0) {
                    setTimeout(triggerFocus, delay);
                }
                else {
                    triggerFocus();
                }
            }
        };
        window.addEventListener('keydown', handler);
        return () => window.removeEventListener('keydown', handler);
    }, [shouldFocus, elementOrRef, triggerKey, delay]);
};
export default useElementFocus;
//# sourceMappingURL=useElementFocus.js.map